﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.VariableVector {
  [Item("VariableVectorManipulator", "Description missing")]
  [StorableClass]
  public abstract class VariableVectorManipulator : SingleSuccessorOperator, IVariableVectorManipulator, IStochasticOperator {

    #region parameters
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<VariableVector> ChildParameter {
      get { return (ILookupParameter<VariableVector>)Parameters["Child"]; }
    }
    public ILookupParameter<VariableVectorInput> FetchedInputParameter {
      get { return (ILookupParameter<VariableVectorInput>)Parameters["Input"]; }
    }
    public IValueLookupParameter<IVariableVectorClassificationProblemData> ProblemDataParameter {
      get { return (IValueLookupParameter<IVariableVectorClassificationProblemData>)Parameters["ProblemData"]; }
    }
    #endregion

    [StorableConstructor]
    protected VariableVectorManipulator(bool deserializing) : base(deserializing) { }
    protected VariableVectorManipulator(VariableVectorManipulator original, Cloner cloner)
      : base(original, cloner) {
    }
    public VariableVectorManipulator()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random"));
      Parameters.Add(new LookupParameter<VariableVector>("Child"));
      Parameters.Add(new LookupParameter<VariableVectorInput>("Input"));
      Parameters.Add(new ValueLookupParameter<IVariableVectorClassificationProblemData>("ProblemData", ""));
    }

    public sealed override IOperation Apply() {
      ChildParameter.ActualValue = Manipulate(RandomParameter.ActualValue, FetchedInputParameter.ActualValue, ChildParameter.ActualValue, ProblemDataParameter.ActualValue.SpreadPercentageParameter.Value.Value);
      return base.Apply();
    }

    protected abstract VariableVector Manipulate(IRandom random, VariableVectorInput input, VariableVector child, double spreadPercentage);
  }
}
